		TITLE	Character set compress utility

dos		EQU 21h			; MSDOS int
vdu		EQU 10h			; Video int
kbd		EQU 16h			; Kbd int

Set_Block	EQU 4Ah
Allocate_Mem	EQU 48h
Deallocate_Mem	EQU 49h

OpenFile	EQU 3Dh
CreateFile	EQU 3Ch
CloseFile	EQU 3Eh
ReadFromFile	EQU 3Fh
WriteToFile	EQU 40h
ModAttrs	EQU 43h
GetVersion	EQU 30h
DeleteFile	EQU 41h
MovePointer	EQU 42h

terminate	EQU 4Ch
dos		EQU 21h
display_char	EQU 02h
dir_con_input	EQU 07h
prt_stg		EQU 09h
srch_fst	EQU 0011h
srch_nxt	EQU 0012h
set_dta		EQU 001Ah

cTab		equ 09h			; ASCII defs
cLF		equ 0Ah			;
cCret		equ 0Dh			;
cEOF		equ 1Ah			;

No_Lines	equ 14			; EGA characters

My_Code		SEGMENT PARA 'Code'

		ASSUME CS:My_Code

Start:		mov ax,StackSeg		; get Stack Segment Origin
		mov SS,ax		; and initialize the SS register
		mov SP,offset TOS	; set SP to Top Of Stack

		call Deallocate		; Free the rest of the RAM
		jc Error		; Fail

		cmp es:byte ptr [80H],0	; Any parameters ?
		jz Usage		; No, show usage message

		mov si,81h		; Start of parameter

		call Skip		; Skip spaces
		jz Usage		; EOL

		call Read_Source	; Get a source file
		jc Error		; Fail

		call Skip		; Skip spaces
		jz Usage		; EOL

		call Open_Object	; Create an output file
		jc Error		; Fail

		mov si,0		; Ptr to source data

		mov es,cs:Source_Base	; Set up segment

		mov di,offset Out_Buffer; Point at buffer

		mov cx,256		; Do this many characters

Char_Lp:	push cx			; Save cnt

		mov cx,No_Lines		; Lines per character

CC_Lp:		push cx			; Save cnt

		call Compress_Line	; Write pixel line

		pop cx			; Restore cnt
		loop CC_Lp		; Loop

		mov ax,si		; Get addr

		ror ax,1		;
		ror ax,1		;
		ror ax,1		;
		ror ax,1		;

		and ax,0FFFh		; Mask

		mov cx,es		; Step segment
		add cx,ax		;
		mov es,cx		;

		and si,000Fh		; Offset

		add si,2		; Skip CRLF

		pop cx			; Restore cnt

		loop Char_Lp		; Loop

		; Now write buffer

		push ds			; Save seg

		mov cx,7168		; Char length
		mov dx,offset Out_Buffer; Ptr

		mov ah,WriteToFile	; Write data
		mov bx,cs:Object_Handle	; Handle

		push cs			; Set up seg
		pop ds			;

		int dos			; Write data

		pop ds			; Restore

		call Close_Object	; Close file

		jmp Quit		; Done

Usage:		push cs			; Set up segment
		pop ds			;

		mov dx,offset sUsage	; No parameters
		mov ah,prt_stg		;
		int dos			;

		jmp Quit		; Done

Error:		push cs			; Set up segment
		pop ds			;

		mov dx,offset sError	; Some sort of failure
		mov ah,prt_stg		;
		int dos			;

Quit:		mov ah,terminate	; Done
		xor al,al		; Terminate
		int dos			;

Skip1:		inc si			; Step ptr

Skip:		mov al,es:[si]		; Get next char

		cmp al,cTab		; Skip if tab
		jz Skip1		;

		cmp al," "		; Skip if space
		jz Skip1		;

		cmp al,cCret		; End of string ?
		ret			; Done

Compress_Line:	add si,2		; Skip two tabs

		call Read_Eight		; Read 8 pixels

		mov cs:[di],al		; Store byte

		call Read_Eight		; Read 8 pixels

		mov cs:3584[di],al	; Store byte

		inc di			; Step ptr

		add si,2		; Skip CRLF

		ret			; Done
		
Read_Eight:	mov cx,8		; 16 chars

CL_Lp:		mov ah,es:[si]		; read char
		inc si			; Skip char

		cmp ah," "		; Space ?
		je CL_Lp		; Yes, ignore

		cmp ah,cTab		; Tab ?
		je CL_Lp		; Yes, ignore

		cmp ah,"."		; Pixel off ?
		je CL_1			; Yes (carry clear)

		stc			; Carry set

CL_1:		rcl al,1		; Shift in bit

		loop CL_Lp		; Loop

		ret			; Done one byte

Open_Object:	mov di,offset Object_FS	; Use this buffer

		call Get_FileName	; Read a name from the command line
		jc RS_Fail		; Can't, done

		mov dx,offset Object_FS	; Ptr into dx

		push ds			; Save segment

		push cs			; Set correct segment for dos call
		pop ds			;

		mov cx,0		; R/W access

		mov ah,CreateFile	; Call dos
		int dos			;

		pop ds			; Restore

		jc RS_Fail		; Can't (use nearest)

		mov cs:Object_Handle,ax	; Save handle

		clc			; No errors
		ret			; Done

Close_Object:	mov bx,CS:Object_Handle	; Close it
		mov ah,CloseFile	;
		int dos			;

		ret			;

RS_Fail:	stc			; Assert fail
		ret			; Done, error

Read_Source:	XOR ax,ax		; Clear handle and base
		MOV CS:Source_Handle,ax	;
		MOV CS:Source_Base,ax	;

		MOV di,offset Buffer	;
		CALL Get_FileName	;

		MOV dx,di		; Ptr into dx

		MOV al,00h		; Read only

		push cs			; Get segment
		pop ds			;

		MOV ah,OpenFile		; Call dos
		INT dos			;

		JC RS_Fail		; Can't (use nearest)

		MOV CS:Source_Handle,ax	; Save handle

		MOV bx,ax		; Get length

		XOR cx,cx		; Zero
		XOR dx,dx		;

		MOV al,2		; From end
	
		MOV ah,MovePointer	;
		INT dos			;

		JC RS_Fail_Close	; Can't

		CMP dx,16		; Less than 1 Mbyte
		JAE RS_Fail_Close	; Won't fit

		SHR dx,1		; Calc no. paras
		RCR ax,1		;
		SHR dx,1		;
		RCR ax,1		;
		SHR dx,1		;
		RCR ax,1		;
		SHR dx,1		;
		RCR ax,1		;

		INC ax			; Assume some more

		MOV bx,ax		; Allocate this much ram

		MOV ah,Allocate_Mem	;
		INT dos			;
		JC RS_Fail_Close	; Can't 

		MOV CS:Source_Base,ax	; Save source base seg

		XOR cx,cx		; Rewind file
		XOR dx,dx		;

		MOV bx,CS:Source_Handle	;

		MOV al,0		;

		MOV ah,MovePointer	;
		INT dos			;

		MOV bx,CS:Source_Handle	; Get file handle

		PUSH ds			; Save data seg

		MOV ax,CS:Source_Base	; Read the file to here
		MOV ds,ax		;

RS_ReadLp:	MOV dx,0		; Offset
		MOV cx,8000h		; 32 k at a time

		MOV ah,ReadFromFile	; Read it
		INT dos			;

		JC RS_Fail_Read		; Error, close it etc

		MOV dx,ds		; Step data seg
		ADD dx,800h		;
		MOV ds,dx		;

		CMP ax,8000h		; Read less ?
		JE RS_ReadLp		; No, might be more to read

		MOV bx,ax		; Point at it

		MOV dx,ds		;
		SUB dx,800h		;
		MOV ds,dx		;

		MOV al,cEOF		; Terminate
		MOV [bx],al		;

		POP ds			;

		MOV bx,CS:Source_Handle	; Close it
		MOV ah,CloseFile	;
		INT dos			;

		CLC			; No error
		RET			;

RS_Fail_Read:	pop ds			; Fail to here, and close . .

RS_Fail_Close:	push ax			; Save error code

		mov bx,CS:Source_Handle	; Close it
		mov ah,CloseFile	;
		int dos			;

		pop ax			;

		stc			; Fail
		ret			;

Get_FileName:	push di			; Save ptr

		mov cs:byte ptr [di],0	; Kill old one

		call Skip		; Skip past spaces
		jz GFN_Fail		; EOL		

GFN_Lp:		mov al,es:[si]		; Get next char

		cmp al," "		; End ?
		je GFN_Done		; Yes

		cmp al,cCret		; End ?
		je GFN_Done		; Yes

		inc si			; Skip it

		mov cs:[di],al		; Save it
		inc di			;

		mov cs:byte ptr [di],0	; Term

		jmp GFN_Lp		; Loop

GFN_Done:	pop di			; Restore ptr

		cmp cs:byte ptr [di],0	; Any chars ?
		jz GFN_Fail		; No, fail

		clc			; Assert OK
		ret			; Done

GFN_Fail:	pop di			; Done

		stc			; Assert fail
		ret			;

Deallocate:	MOV bx,offset TOS	; Point at top of ss

		MOV cl,4		; Convert to paragraphs
		SHR bx,cl		;
		INC bx			; add one for luck

		MOV cx,ss		; Get stack segment
		ADD bx,cx		; Add stack segment
		
		MOV cx,es		; Get extra segment
		SUB bx,cx		; Subtract it

		; bx contains no. of paragraphs from es to the top of ss

		MOV ah,Set_Block	; Deallocate surplus RAM
		INT dos			;

		MOV al,9		; "Internal memory"
		RET			;

sUsage:		DB 'Usage :-',cCret,cLF,cLF
		DB 'CMP_CHAR [ Source file spec ] [ Output char spec ]'
		DB cCret,cLF
		DB 'Converts character set definitions to binary files'
sCRLF		DB cCret,cLF,cLF,'$'

sError		DB 'Some sort of error has occured',cCret,cLF,'$'

Buffer:					; Use it as a buffer later
Source_FS	DB 80 dup (?)		; Source file spec.

Source_Handle	DW 0			; Source File handle

Source_Base	DW 0			; Source buffer segment
Source_End	DW 0			; End of source file

Object_FS	DB 80 dup (?)		;

Object_Handle	DW 0			;

Out_Buffer	DB 7200 dup (?)		; Compressed char set

My_Code		ENDS			; Done

StackSeg	SEGMENT STACK

		DW 100 dup (?)		; Some stack space
TOS		EQU THIS WORD		;

StackSeg	ENDS

		END Start		; Execute address
