; All references to the player's character use the male pronoun for brevity,
; rather than the cumbersome "(s)he" form. Women are, however, still perfectly
; free to feel persecuted if they so wish.

Screen_Width	equ 40			; Screen dimentions
Screen_Height	equ 24			;

Kbd_Status	EQU 64H			; Port address
Kbd_Data	EQU 60H			;

Int_Ctrl	EQU 20H			; Command
Int_Data	EQU 21H			; Data (Masks)

EOI		EQU 20H			; Command to 8259A

kZ		equ 2Ch			; Key scan codes Left
kX		equ 2Dh			; Right
kN		equ 31h			; Up
kM		equ 32h			; Down
kQ		equ 10h			; ?
kA		equ 1Eh			; ?
kK		equ 25h			; Keep
kD		equ 20h			; Drop
kL		equ 26h			; Lightning
kF		equ 21h			; Fire
kH		equ 23h			; Heal
kSpace		equ 39h			; Justify
kShiftL		equ 2Ah			; ?
kShiftR		equ 36h			; ?
kEsc		equ 01h			; Exit

;  The following maze sizes are chosen to make the conversion between maze and
; screen addresses as fast as possible, ie there are 40 paragraphs per line of
; maze, 10 paragraphs per line of screen, thus a divide by 4 converts from maze
; Y's to screen Y's.

; DO NOT ALTER FROM HERE DOWN

breadth		equ 88			; Size of small maze in rooms
height		equ 80			;

BM_Max_X	equ (breadth * 7) + 1	; Size of BM in cells
BM_Max_Y	equ (height * 7) + 1	;

BM_Y_Stride	equ 40			; Segment stride between rows (paragraphs)
BM_Width	equ BM_Y_Stride * 16	; Offset stride between rows

BM_Size		equ BM_Y_Stride * (BM_Max_Y+2) ; Size in paragraphs

; TO HERE !

No_pillars	equ 1500		; (Maximum) number of filled cells

no_go_char	equ 080h
room_char	equ 040h
pillar_char	equ 020h

No_looks	EQU 30

Man_SP		equ 12

Medicine_C	equ 300

Look_Range	equ 7 * 12		; Sight range in cells

Sword_Dam	equ 20			; Wounds inflicted
Light_Dam	equ 20			;
Fire_Dam	equ 30			;

Light_Cost	equ 10			; Amount of magic required

Prob_MFL	equ (20	*256)/100	; Prob. of thing firing light = 20%

No_objects	equ 3000

; Monster offsets

mDelay		equ 0			; Delay
mSpeed		equ 1			; Initial value

mUnderChar	equ 2			; Char we replaced
mRingsHeld	equ 3			; Rings/Key held (bit 7 = Key)

mSeg		equ 4			; Segment
mOff		equ 6			; Offset

mSightpMan0	equ 8			; Ptr to line defn for man 0
mSightpMan1	equ 10			;			   1

mpMove		equ 12			; Move routine

mWounds		equ 14			; Damage sustained
mMagic		equ 15			; Magic possessed

mUnderSword	equ 16			; Char behind sword

mDirection	equ 17			; Current direction

mFlags		equ 18			; Flags

		; Bit 	7 = (Both)   Dead
		;	6 = (Things) Steaming
		;	5 =
		;	4 =
		;	3 =
		;	2 = (Both)   0 = THING : 1 = MAN
		;	1 = (Both)   Sword drawn
		;	0 = (Man)    Has moved

mChar		equ 19			; Character

mSouls		equ 20			; The monsters' soul count

mProjCnt	equ 22;24,26		; Projectiles aimed at it
					; By man0, Man1 and things

mSwordOff	equ 28			; Sword ptr
mSwordSeg	equ 30			;

mnSwag		equ 32			; Amount of swag carried

mWounds_Frac	equ 33;34		; Fractions of wounds
mSteamCnt	equ mWounds_Frac	; OR steam cnt

mWeap_Cost	equ 35			; Current debt from using weapons

mType		equ 36			; 0/1 for the men, 2 for the rest

; At this point we have some store

mData		equ 37			; Start here

lData		equ 0			; For this long

		; NONE as yet - feel free to change, SMB

mSwag		equ mData + lData	; Objects carried

lMon		equ mSwag + 16		; The length of the data

nMons		equ 200			; Number of nasties

Monster_size	equ ((nMons+1)*lMon+15)/16 ; Size of monster list

; Projectile offsets

ppNext		equ 0			; ppNext MUST BE at offset 0

pOff		equ 2			; pMaze
pSeg		equ 4			;

ppOwner		equ 6			;
ppSpeedTab	equ 8	  		;

ppMove		equ 10			;

pdHoriz		equ 12;14		;
pdDiag		equ 16;18		;

p_e		equ 20			;
p_c1		equ 21			;
p_c2		equ 22			;

pRange		equ 23			;

ppTarget	equ 24			;

pDelay		equ 26			; Timing
pSpeed		equ 27  		;

pSpeedDel	equ 28			;
pFirepCnt	equ 29			;

pStartCnt       equ 30                  ;
pUnderChar      equ 31                  ;

pFlags          equ 32                  ;
pChar           equ 33                  ;

lProj		equ 34			; The length of the data

nProjs		equ 300			; Number of nasties

Proj_size	equ ((nProjs+1)*lProj+15)/16 ; Size of proj list

; Sight offsets ( used when drawing lines of sight from the man to the things )

sdHoriz		equ 0;2			; Off/Seg of horiz disp
sdDiag		equ 4;6			;	     diag disp

s_e		equ 8			; Error term
s_c1		equ 9			; Constants
s_c2		equ 10			;

sRange		equ 11			; Distance away

spTarget	equ 12;13		; Ptr to thing aimed at

lSight		equ 14			; Length of a sighting

; ??

ttr		EQU 000h		;

; Character numbers

cRings		equ 088h		;
cKey		equ 08Fh		;

cWalls		equ 090h		;

cLight		equ 0A2h		;

cMan		equ 0A8h		;

cTreasure	equ 0B0h		;
cMagic		equ 0C0h		;

cMonsters	equ 0D0h		;

cDoors		equ 0DCh		;

cDead_Magic	equ 0E0h		;

cSwords		equ 0E4h		;
cDead_ExMagic	equ 0F6h		;

cDead_Sword	equ 0DEh		;
cDead_Arrow	equ 0DFh		;
cDead_Man	equ 0F4h;F5		;
